  using System;
  using System.Collections.Generic;
  using System.Linq;
  using Michigan.Ods.Sdk.Helpers;
  using Michigan.Ods.Sdk.Models;
  using RestSharp;
  using System.Net;
  using System.Text;
  
  namespace Michigan.Ods.Sdk.Api {
    public class SectionsApi {

      private readonly IRestClient client;

      public SectionsApi(IRestClient client)
      {
            this.client = client;
      }
      
      /// <summary>
      /// Retrieves resources based with paging capabilities (using the &quot;Get All&quot; pattern). This GET operation provides access to resources using the &quot;Get All&quot; pattern. In this version of the API there is support for paging.
      /// </summary>
      /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
      /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
      /// <returns></returns>
      public IRestResponse<List<Section>> GetSectionsAll(int? offset, int? limit) {
        var request = new RestRequest("/sections", Method.GET);
        request.RequestFormat = DataFormat.Json;

        if (offset != null){
          request.AddParameter("offset", offset);
		}
        if (limit != null){
          request.AddParameter("limit", limit);
		}
        var response = client.Execute<List<Section>>(request);

        return response;
      }
      /// <summary>
      /// Retrieves resources matching values of an example resource (using the &quot;Get By Example&quot; pattern). This GET operation provides access to resources using the &quot;Get by Example&quot; search pattern.  The values of any properties of the resource that are specified will be used to return all matching results (if it exists).
      /// </summary>
      /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
      /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
      /// <param name="schoolId">School Identity Column</param>
      /// <param name="classPeriodName">An indication of the portion of a typical daily session in which students receive instruction in a specified subject (e.g., morning, sixth period, block period, or AB schedules).   NEDM: Class Period</param>
      /// <param name="classroomIdentificationCode">A unique number or alphanumeric code assigned to a room by a school, school system, state, or other agency or entity.</param>
      /// <param name="localCourseCode">The local code assigned by the LEA or Campus that identifies the organization of subject matter and related learning experiences provided for the instruction of students.</param>
      /// <param name="termDescriptor">A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.</param>
      /// <param name="schoolYear">The identifier for the school year.</param>
      /// <param name="uniqueSectionCode">A unique identifier for the Section, that is defined by the classroom, the subjects taught, and the instructors that are assigned.</param>
      /// <param name="sequenceOfCourse">When a Section is part of a sequence of parts for a course, the number if the sequence. If the course has only one part, the value of this Section attribute should be 1.</param>
      /// <param name="educationalEnvironmentType">The setting in which a child receives education and related services; for example:  Center-based instruction  Home-based instruction  Hospital class  Mainstream  Residential care and treatment facility  ....</param>
      /// <param name="mediumOfInstructionType">The media through which teachers provide instruction to students and students and teachers communicate about instructional matters; for example:  Technology-based instruction in classroom  Correspondence instruction  Face-to-face instruction  Virtual/On-line Distance learning  Center-based instruction  ...</param>
      /// <param name="populationServedType">The population for which the course was designed; for example:  Bilingual students  Remedial education students  Gifted and talented students  Career and Technical Education students  Special education students  ....</param>
      /// <param name="availableCreditType">The type of credits or units of value awarded for the completion of a course.</param>
      /// <param name="availableCreditConversion">Conversion factor that when multiplied by the number of credits is equivalent to Carnegie units.</param>
      /// <param name="instructionLanguageDescriptor">A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.</param>
      /// <param name="availableCredits">Credits or units of value awarded for the completion of a course.</param>
      /// <param name="id"></param>
      /// <returns></returns>
      public IRestResponse<List<Section>> GetSectionsAll(int? offset, int? limit, int? schoolId, string classPeriodName, string classroomIdentificationCode, string localCourseCode, string termDescriptor, int? schoolYear, string uniqueSectionCode, int? sequenceOfCourse, string educationalEnvironmentType, string mediumOfInstructionType, string populationServedType, string availableCreditType, double? availableCreditConversion, string instructionLanguageDescriptor, double? availableCredits, string id) {
        var request = new RestRequest("/sections", Method.GET);
        request.RequestFormat = DataFormat.Json;

        if (offset != null){
          request.AddParameter("offset", offset);
		}
        if (limit != null){
          request.AddParameter("limit", limit);
		}
        if (schoolId != null){
          request.AddParameter("schoolId", schoolId);
		}
        if (classPeriodName != null){
          request.AddParameter("classPeriodName", classPeriodName);
		}
        if (classroomIdentificationCode != null){
          request.AddParameter("classroomIdentificationCode", classroomIdentificationCode);
		}
        if (localCourseCode != null){
          request.AddParameter("localCourseCode", localCourseCode);
		}
        if (termDescriptor != null){
          request.AddParameter("termDescriptor", termDescriptor);
		}
        if (schoolYear != null){
          request.AddParameter("schoolYear", schoolYear);
		}
        if (uniqueSectionCode != null){
          request.AddParameter("uniqueSectionCode", uniqueSectionCode);
		}
        if (sequenceOfCourse != null){
          request.AddParameter("sequenceOfCourse", sequenceOfCourse);
		}
        if (educationalEnvironmentType != null){
          request.AddParameter("educationalEnvironmentType", educationalEnvironmentType);
		}
        if (mediumOfInstructionType != null){
          request.AddParameter("mediumOfInstructionType", mediumOfInstructionType);
		}
        if (populationServedType != null){
          request.AddParameter("populationServedType", populationServedType);
		}
        if (availableCreditType != null){
          request.AddParameter("availableCreditType", availableCreditType);
		}
        if (availableCreditConversion != null){
          request.AddParameter("availableCreditConversion", availableCreditConversion);
		}
        if (instructionLanguageDescriptor != null){
          request.AddParameter("instructionLanguageDescriptor", instructionLanguageDescriptor);
		}
        if (availableCredits != null){
          request.AddParameter("availableCredits", availableCredits);
		}
        if (id != null){
          request.AddParameter("id", id);
		}
        var response = client.Execute<List<Section>>(request);

        return response;
      }
      /// <summary>
      /// Retrieves a specific resource using the values of the resource's natural key (using the &quot;Get By Key&quot; pattern). This GET operation provides access to resources using the &quot;Get by Key&quot; search pattern. The values of the natural key of the resource must be fully specified, and the service will return the matching result (if it exists).
      /// </summary>
      /// <param name="schoolId">School Identity Column</param>
      /// <param name="classPeriodName">An indication of the portion of a typical daily session in which students receive instruction in a specified subject (e.g., morning, sixth period, block period, or AB schedules).   NEDM: Class Period</param>
      /// <param name="classroomIdentificationCode">A unique number or alphanumeric code assigned to a room by a school, school system, state, or other agency or entity.</param>
      /// <param name="localCourseCode">The local code assigned by the LEA or Campus that identifies the organization of subject matter and related learning experiences provided for the instruction of students.</param>
      /// <param name="termDescriptor">A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.</param>
      /// <param name="schoolYear">The identifier for the school year.</param>
      /// <param name="uniqueSectionCode">A unique identifier for the Section, that is defined by the classroom, the subjects taught, and the instructors that are assigned.</param>
      /// <param name="sequenceOfCourse">When a Section is part of a sequence of parts for a course, the number if the sequence. If the course has only one part, the value of this Section attribute should be 1.</param>
      /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
      /// <returns></returns>
      public IRestResponse<Section> GetSectionByKey(int? schoolId, string classPeriodName, string classroomIdentificationCode, string localCourseCode, string termDescriptor, int? schoolYear, string uniqueSectionCode, int? sequenceOfCourse, string IfNoneMatch) {
        var request = new RestRequest("/sections", Method.GET);
        request.RequestFormat = DataFormat.Json;

        // verify required params are set
        if (schoolId == null || classPeriodName == null || classroomIdentificationCode == null || localCourseCode == null || termDescriptor == null || schoolYear == null || uniqueSectionCode == null || sequenceOfCourse == null ) {
           throw new ArgumentException("API method call is missing required parameters");
        }
        if (schoolId != null){
          request.AddParameter("schoolId", schoolId);
		}
        if (classPeriodName != null){
          request.AddParameter("classPeriodName", classPeriodName);
		}
        if (classroomIdentificationCode != null){
          request.AddParameter("classroomIdentificationCode", classroomIdentificationCode);
		}
        if (localCourseCode != null){
          request.AddParameter("localCourseCode", localCourseCode);
		}
        if (termDescriptor != null){
          request.AddParameter("termDescriptor", termDescriptor);
		}
        if (schoolYear != null){
          request.AddParameter("schoolYear", schoolYear);
		}
        if (uniqueSectionCode != null){
          request.AddParameter("uniqueSectionCode", uniqueSectionCode);
		}
        if (sequenceOfCourse != null){
          request.AddParameter("sequenceOfCourse", sequenceOfCourse);
		}
        request.AddHeader("If-None-Match", IfNoneMatch);
        var response = client.Execute<Section>(request);

        return response;
      }
      /// <summary>
      /// Creates or updates resources based on the natural key values of the supplied resource. The POST operation can be used to create or update resources. In database terms, this is often referred to as an &quot;upsert&quot; operation (insert + update).  Clients should NOT include the resource &quot;id&quot; in the JSON body because it will result in an error (you must use a PUT operation to update a resource by &quot;id&quot;). The web service will identify whether the resource already exists based on the natural key values provided, and update or create the resource appropriately.
      /// </summary>
      /// <param name="body">The JSON representation of the &quot;section&quot; resource to be created or updated.</param>
      /// <returns></returns>
      public IRestResponse PostSections(Section body) {
        var request = new RestRequest("/sections", Method.POST);
        request.RequestFormat = DataFormat.Json;

        // verify required params are set
        if (body == null ) {
           throw new ArgumentException("API method call is missing required parameters");
        }
        request.AddBody(body);
        var response = client.Execute(request);

        var location = response.Headers.FirstOrDefault(x => x.Name == "Location");

        if (location != null && !string.IsNullOrWhiteSpace(location.Value.ToString()))
          {
              body.id = location.Value.ToString().Split('/').Last();
          }
        return response;
      }
      /// <summary>
      /// Retrieves a specific resource using the resource's identifier (using the &quot;Get By Id&quot; pattern). This GET operation retrieves a resource by the specified resource identifier.
      /// </summary>
      /// <param name="id">A resource identifier specifying the resource to be retrieved.</param>
      /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
      /// <returns></returns>
      public IRestResponse<Section> GetSectionsById(string id, string IfNoneMatch) {
        var request = new RestRequest("/sections/{id}", Method.GET);
        request.RequestFormat = DataFormat.Json;

        request.AddUrlSegment("id", id);
        // verify required params are set
        if (id == null ) {
           throw new ArgumentException("API method call is missing required parameters");
        }
        request.AddHeader("If-None-Match", IfNoneMatch);
        var response = client.Execute<Section>(request);

        return response;
      }
      /// <summary>
      /// Updates or creates a resource based on the resource identifier. The PUT operation is used to update or create a resource by identifier.  If the resource doesn't exist, the resource will be created using that identifier.  Additionally, natural key values cannot be changed using this operation, and will not be modified in the database.  If the resource &quot;id&quot; is provided in the JSON body, it will be ignored as well.
      /// </summary>
      /// <param name="id">A resource identifier specifying the resource to be updated.</param>
      /// <param name="IfMatch">The ETag header value used to prevent the PUT from updating a resource modified by another consumer.</param>
      /// <param name="body">The JSON representation of the &quot;section&quot; resource to be updated.</param>
      /// <returns></returns>
      public IRestResponse PutSection(string id, string IfMatch, Section body) {
        var request = new RestRequest("/sections/{id}", Method.PUT);
        request.RequestFormat = DataFormat.Json;

        request.AddUrlSegment("id", id);
        // verify required params are set
        if (id == null || body == null ) {
           throw new ArgumentException("API method call is missing required parameters");
        }
        request.AddHeader("If-Match", IfMatch);
        request.AddBody(body);
        var response = client.Execute(request);

        var location = response.Headers.FirstOrDefault(x => x.Name == "Location");

        if (location != null && !string.IsNullOrWhiteSpace(location.Value.ToString()))
          {
              body.id = location.Value.ToString().Split('/').Last();
          }
        return response;
      }
      /// <summary>
      /// Deletes an existing resource using the resource identifier. The DELETE operation is used to delete an existing resource by identifier.  If the resource doesn't exist, an error will result (the resource will not be found).
      /// </summary>
      /// <param name="id">A resource identifier specifying the resource to be deleted.</param>
      /// <param name="IfMatch">The ETag header value used to prevent the DELETE from removing a resource modified by another consumer.</param>
      /// <returns></returns>
      public IRestResponse DeleteSectionById(string id, string IfMatch) {
        var request = new RestRequest("/sections/{id}", Method.DELETE);
        request.RequestFormat = DataFormat.Json;

        request.AddUrlSegment("id", id);
        // verify required params are set
        if (id == null ) {
           throw new ArgumentException("API method call is missing required parameters");
        }
        request.AddHeader("If-Match", IfMatch);
        var response = client.Execute(request);

        return response;
      }
      /// <summary>
        ///Retrieves a specific resource using the values of the resource's natural key (using the &quot;Get By Key&quot; pattern). This GET operation provides access to resources using the &quot;Get by Key&quot; search pattern. The values of the natural key of the resource must be fully specified, and the service will return the matching result (if it exists).
        ///</summary>
        /// <param name="schoolId">School Identity Column</param>
        /// <param name="classPeriodName">An indication of the portion of a typical daily session in which students receive instruction in a specified subject (e.g., morning, sixth period, block period, or AB schedules).   NEDM: Class Period</param>
        /// <param name="classroomIdentificationCode">A unique number or alphanumeric code assigned to a room by a school, school system, state, or other agency or entity.</param>
        /// <param name="localCourseCode">The local code assigned by the LEA or Campus that identifies the organization of subject matter and related learning experiences provided for the instruction of students.</param>
        /// <param name="termDescriptor">A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.</param>
        /// <param name="schoolYear">The identifier for the school year.</param>
        /// <param name="uniqueSectionCode">A unique identifier for the Section, that is defined by the classroom, the subjects taught, and the instructors that are assigned.</param>
        /// <param name="sequenceOfCourse">When a Section is part of a sequence of parts for a course, the number if the sequence. If the course has only one part, the value of this Section attribute should be 1.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        ///<returns></returns>
        public IRestResponse PatchGetSectionByKey(int? schoolId, string classPeriodName, string classroomIdentificationCode, string localCourseCode, string termDescriptor, int? schoolYear, string uniqueSectionCode, int? sequenceOfCourse, string IfNoneMatch, Section patchSection)
        {
            // verify required params are set
            if (schoolId == null || classPeriodName == null || classroomIdentificationCode == null || localCourseCode == null || termDescriptor == null || schoolYear == null || uniqueSectionCode == null || sequenceOfCourse == null ) {
                throw new ArgumentException("API method call is missing required parameters");
            }
            var sdkUsageRecord = new SdkUsageRecord();
            var jsonSerializer = new RestSharp.Serializers.JsonSerializer();
            var statisticsTracker = new StatisticsTracker();

            var getResponse = this.GetSectionByKey( schoolId, classPeriodName, classroomIdentificationCode, localCourseCode, termDescriptor, schoolYear, uniqueSectionCode, sequenceOfCourse, IfNoneMatch);

            if (!(getResponse.StatusCode == HttpStatusCode.OK || getResponse.StatusCode == HttpStatusCode.Found))
                return getResponse;

            if (statisticsTracker.ShouldTrackStatistics)
            {
                sdkUsageRecord.DateTimeStamp = DateTime.Now;
                sdkUsageRecord.HttpMethod = "PatchGetSectionByKey";
                sdkUsageRecord.Url = "/sections";
                var serializedGetObject = jsonSerializer.Serialize(getResponse.Data);
                sdkUsageRecord.GetSizeInBytes = Encoding.UTF8.GetByteCount(serializedGetObject);
                var serializedPatchObject = jsonSerializer.Serialize(patchSection);
                sdkUsageRecord.PatchSizeInBytes = Encoding.UTF8.GetByteCount(serializedPatchObject);
            }

            new PatchApplier().Execute(getResponse.Data, patchSection);

            var putResponse = PutSection( getResponse.Data.id, null, getResponse.Data ); 
            if (statisticsTracker.ShouldTrackStatistics)
            {
                var serializedPutObject = jsonSerializer.Serialize(getResponse.Data);
                sdkUsageRecord.PutSizeInBytes = Encoding.UTF8.GetByteCount(serializedPutObject);
                statisticsTracker.TrackIfEnabled(sdkUsageRecord);
            }
            return putResponse;
        }
        /// <summary>
        ///Retrieves a specific resource using the resource's identifier (using the &quot;Get By Id&quot; pattern). This GET operation retrieves a resource by the specified resource identifier.
        ///</summary>
        /// <param name="id">A resource identifier specifying the resource to be retrieved.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        ///<returns></returns>
        public IRestResponse PatchGetSectionsById(string id, string IfNoneMatch, Section patchSection)
        {
            // verify required params are set
            if (id == null ) {
                throw new ArgumentException("API method call is missing required parameters");
            }
            var sdkUsageRecord = new SdkUsageRecord();
            var jsonSerializer = new RestSharp.Serializers.JsonSerializer();
            var statisticsTracker = new StatisticsTracker();

            var getResponse = this.GetSectionsById( id, IfNoneMatch);

            if (!(getResponse.StatusCode == HttpStatusCode.OK || getResponse.StatusCode == HttpStatusCode.Found))
                return getResponse;

            if (statisticsTracker.ShouldTrackStatistics)
            {
                sdkUsageRecord.DateTimeStamp = DateTime.Now;
                sdkUsageRecord.HttpMethod = "PatchGetSectionsById";
                sdkUsageRecord.Url = "/sections/{id}";
                var serializedGetObject = jsonSerializer.Serialize(getResponse.Data);
                sdkUsageRecord.GetSizeInBytes = Encoding.UTF8.GetByteCount(serializedGetObject);
                var serializedPatchObject = jsonSerializer.Serialize(patchSection);
                sdkUsageRecord.PatchSizeInBytes = Encoding.UTF8.GetByteCount(serializedPatchObject);
            }

            new PatchApplier().Execute(getResponse.Data, patchSection);

            var putResponse = PutSection( getResponse.Data.id, null, getResponse.Data ); 
            if (statisticsTracker.ShouldTrackStatistics)
            {
                var serializedPutObject = jsonSerializer.Serialize(getResponse.Data);
                sdkUsageRecord.PutSizeInBytes = Encoding.UTF8.GetByteCount(serializedPutObject);
                statisticsTracker.TrackIfEnabled(sdkUsageRecord);
            }
            return putResponse;
        }
        }
}
