using System;
using System.Collections.Generic;
using System.Linq;
using Michigan.Ods.Sdk.Api;
using Michigan.Ods.Sdk.Helpers;
using Michigan.Ods.Sdk.Models;
using RestSharp;

namespace Michigan.Ods.Sdk.Composites.FoodService
{
    public class FoodServiceApi
    {
        private readonly StudentsApi _studentsApi;
        private readonly StudentSchoolAssociationsApi _studentSchoolAssociationsApi;
        private readonly StudentSectionAssociationsApi _studentSectionAssociationsApi;
        private readonly StaffSectionAssociationsApi _staffSectionAssociationsApi;
        private readonly StaffsApi _staffsApi;
        private readonly StudentParentAssociationsApi _studentParentAssociationsApi;
        private readonly ParentsApi _parentsApi;

        public FoodServiceApi(IRestClient client)
        {
            _studentsApi = new StudentsApi(client);
            _studentSchoolAssociationsApi = new StudentSchoolAssociationsApi(client);
            _studentSectionAssociationsApi = new StudentSectionAssociationsApi(client);
            _staffSectionAssociationsApi = new StaffSectionAssociationsApi(client);
            _staffsApi = new StaffsApi(client);
            _studentParentAssociationsApi = new StudentParentAssociationsApi(client);
            _parentsApi = new ParentsApi(client);
        }

        /// <summary>
        ///     Retrieves resources based with paging capabilities (using the &quot;Get All&quot; pattern). This GET operation
        ///     provides access to resources using the &quot;Get All&quot; pattern. In this version of the API there is support for
        ///     paging.
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="fields">
        ///     Specifies a subset of properties that should be returned for each entity (e.g. &quot;
        ///     property1,collection1(collProp1,collProp2)&quot;).
        /// </param>
        /// <returns></returns>
        public IRestResponse<List<FoodServiceStudent>> GetFoodServiceStudentsAll(int? offset, int? limit, string fields)
        {
            var studentGetAllResponse = _studentsApi.GetStudentsAll(offset, limit);

            if (!ManualCompositesHelper.IsValidRestResponse(studentGetAllResponse))
                return ManualCompositesHelper.CopyRestResponse(studentGetAllResponse, new List<FoodServiceStudent>());

            return GetManualFoodServiceStudentComposite(studentGetAllResponse.Data, studentGetAllResponse);
        }

        /// <summary>
        ///     Retrieves a specific resource using the resource's identifier (using the &quot;Get By Example&quot; pattern). This
        ///     GET operation provides access to resources using the &quot;Get By Example&quot; pattern. The values of any
        ///     properties of the resource that are specified will be used to return all matching results (if it exists).
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="fields">
        ///     Specifies a subset of properties that should be returned for each entity (e.g. &quot;
        ///     property1,collection1(collProp1,collProp2)&quot;).
        /// </param>
        /// <param name="q">
        ///     Specifies a query filter expression for the request.  Currently only supports range-based queries on
        ///     dates and numbers (e.g. &quot;[2016-03-07..2016-03-10]&quot;).
        /// </param>
        /// <param name="studentUniqueId">A unique alphanumeric code assigned to a student.</param>
        /// <param name="citizenshipStatusType">
        ///     A unique identifier used as Primary Key, not derived from business logic, when
        ///     acting as Foreign Key, references the parent table.
        /// </param>
        /// <param name="birthCountryDescriptor">
        ///     A unique identifier used as Primary Key, not derived from business logic, when
        ///     acting as Foreign Key, references the parent table.
        /// </param>
        /// <param name="limitedEnglishProficiencyDescriptor">
        ///     An indication that the student has sufficient difficulty speaking,
        ///     reading, writing, or understanding the English language, as to require special English Language services.
        /// </param>
        /// <param name="oldEthnicityType">
        ///     Previous definition of Ethnicity combining Hispanic/latino and race:  1 - American
        ///     Indian or Alaskan Native  2 - Asian or Pacific Islander  3 - Black, not of Hispanic origin  4 - Hispanic  5 -
        ///     White, not of Hispanic origin
        /// </param>
        /// <param name="schoolFoodServicesEligibilityDescriptor">
        ///     A unique identifier used as Primary Key, not derived from
        ///     business logic, when acting as Foreign Key, references the parent table.
        /// </param>
        /// <param name="sexType">A person''s gender.</param>
        /// <param name="birthStateAbbreviationType">
        ///     The abbreviation for the name of the state (within the United States) or
        ///     extra-state jurisdiction in which an individual was born.
        /// </param>
        /// <param name="personalTitlePrefix">A prefix used to denote the title, degree, position, or seniority of the person.</param>
        /// <param name="firstName">
        ///     A name given to an individual at birth, baptism, or during another naming ceremony, or through
        ///     legal change.
        /// </param>
        /// <param name="middleName">A secondary name given to an individual at birth, baptism, or during another naming ceremony.</param>
        /// <param name="lastSurname">The name borne in common by members of a family.</param>
        /// <param name="generationCodeSuffix">
        ///     An appendage, if any, used to denote an individual's generation in his family (e.g.,
        ///     Jr., Sr., III).
        /// </param>
        /// <param name="maidenName">The person's maiden name.</param>
        /// <param name="birthDate">The month, day, and year on which an individual was born.</param>
        /// <param name="birthCity">
        ///     The set of elements that capture relevant data regarding a person's birth, including birth date
        ///     and place of birth.
        /// </param>
        /// <param name="dateEnteredUS">For students born outside of the U.S., the date the student entered the U.S.</param>
        /// <param name="multipleBirthStatus">
        ///     Indicator of whether the student was born with other siblings (i.e., twins, triplets,
        ///     etc.)
        /// </param>
        /// <param name="profileThumbnail">ProfileThumbnail.</param>
        /// <param name="hispanicLatinoEthnicity">
        ///     An indication that the individual traces his or her origin or descent to Mexico,
        ///     Puerto Rico, Cuba, Central, and South America, and other Spanish cultures, regardless of race. The term, &quot;
        ///     Spanish origin,&quot; can be used in addition to &quot;Hispanic or Latino.&quot;
        /// </param>
        /// <param name="economicDisadvantaged">
        ///     An indication of inadequate financial condition of an individual's family, as
        ///     determined by family income, number of family members/dependents, participation in public assistance programs,
        ///     and/or other characteristics considered relevant by federal, state, and local policy.
        /// </param>
        /// <param name="displacementStatus">
        ///     Indicates a state health or weather related event that displaces a group of students,
        ///     and may require additional funding, educational, or social services.
        /// </param>
        /// <param name="loginId">The login ID for the user; used for security access control interface.</param>
        /// <param name="birthInternationalProvince">
        ///     For students born outside of the US, the Province or jurisdiction in which an
        ///     individual is born.
        /// </param>
        /// <param name="id"></param>
        /// <param name="birthCountryDescriptor">A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.</param>
        /// <param name="yearOfEntry">(MI-EXT) The four-digit calendar year in which the student first attended school in the United States.</param>
        /// <param name="raceAsOfDate">(MI-EXT) The date of the most recent change in Race.</param>
        /// <param name="schoolFoodServicesEligibilityAsOfDate">(MI-EXT) The date of the most recent change in SchoolFoodServicesEligibility.</param>
        /// <param name="addressAsOfDate">(MI-EXT) The date of the most recent changes to Address.</param>
        /// <param name="homelessDescriptor">A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.</param>
        /// <param name="foodServiceStatusDescriptor">(MI EXT) A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.</param>
        /// <param name="medicalNotes">(MI EXT) Medical notes such as &quot;Allergic to Bee Stings&quot; - For Transportation records.</param>
        /// <returns></returns>
        public IRestResponse<List<FoodServiceStudent>> GetFoodServiceStudentsByExample(int? offset, int? limit,
            string fields, string q, string studentUniqueId, string citizenshipStatusType, string birthCountryDescriptor,
            string limitedEnglishProficiencyDescriptor, string oldEthnicityType,
            string schoolFoodServicesEligibilityDescriptor, string sexType, string birthStateAbbreviationType,
            string personalTitlePrefix, string firstName, string middleName, string lastSurname,
            string generationCodeSuffix, string maidenName, DateTime? birthDate, string birthCity,
            DateTime? dateEnteredUS, bool? multipleBirthStatus, string profileThumbnail, bool? hispanicLatinoEthnicity,
            bool? economicDisadvantaged, string displacementStatus, string loginId, string birthInternationalProvince,
            string id, int? yearOfEntry, DateTime? raceAsOfDate, DateTime? schoolFoodServicesEligibilityAsOfDate, DateTime? addressAsOfDate, string homelessDescriptor, string foodServiceStatusDescriptor, string medicalNotes)
        {
            var studentGetAllResponse = _studentsApi.GetStudentsAll(offset, limit, studentUniqueId, personalTitlePrefix,
                firstName, middleName, lastSurname, generationCodeSuffix, maidenName, sexType, birthDate, birthCity,
                birthStateAbbreviationType, dateEnteredUS, multipleBirthStatus, profileThumbnail,
                hispanicLatinoEthnicity, oldEthnicityType, economicDisadvantaged,
                schoolFoodServicesEligibilityDescriptor, limitedEnglishProficiencyDescriptor, displacementStatus,
                loginId, birthInternationalProvince, citizenshipStatusType, id, birthCountryDescriptor, yearOfEntry,
                raceAsOfDate, schoolFoodServicesEligibilityAsOfDate, addressAsOfDate, homelessDescriptor,
                foodServiceStatusDescriptor, medicalNotes);

            if (!ManualCompositesHelper.IsValidRestResponse(studentGetAllResponse))
                return ManualCompositesHelper.CopyRestResponse(studentGetAllResponse, new List<FoodServiceStudent>());

            return GetManualFoodServiceStudentComposite(studentGetAllResponse.Data, studentGetAllResponse);
        }

        /// <summary>
        ///     Retrieves a specific resource using the resource's identifier (using the &quot;Get By Key&quot; pattern). This GET
        ///     operation provides access to resources using the &quot;Get By Key&quot; pattern. The values of the natural key of
        ///     the resource must be fully specified, and the service will return the matching result (if it exists).
        /// </summary>
        /// <param name="studentUniqueId">A unique alphanumeric code assigned to a student.</param>
        /// <param name="fields">
        ///     Specifies a subset of properties that should be returned for each entity (e.g. &quot;
        ///     property1,collection1(collProp1,collProp2)&quot;).
        /// </param>
        /// <param name="IfNoneMatch">
        ///     The previously returned ETag header value, used here to prevent the unnecessary data transfer
        ///     of an unchanged resource.
        /// </param>
        /// <returns></returns>
        public IRestResponse<FoodServiceStudent> GetFoodServiceStudentsByKey(string studentUniqueId, string fields,
            string IfNoneMatch)
        {
            var studentGetByKeyResponse = _studentsApi.GetStudentByKey(studentUniqueId, IfNoneMatch);

            if (!ManualCompositesHelper.IsValidRestResponse(studentGetByKeyResponse))
                return ManualCompositesHelper.CopyRestResponse<FoodServiceStudent>(studentGetByKeyResponse, null);

            var mappedResponse = GetManualFoodServiceStudentComposite(new List<Student> { studentGetByKeyResponse.Data }, studentGetByKeyResponse);
            return ManualCompositesHelper.CopyRestResponse(mappedResponse, mappedResponse.Data.Single());
        }

        /// <summary>
        /// Retrieves a specific resource using the resource's identifier (using the &quot;Get By Id&quot; pattern). This GET operation provides access to resources using the &quot;Get By Id&quot; pattern.
        /// </summary>
        /// <param name="id"></param>
        /// <param name="fields">Specifies a subset of properties that should be returned for each entity (e.g. &quot;property1,collection1(collProp1,collProp2)&quot;).</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        /// <returns></returns>
        public IRestResponse<FoodServiceStudent> GetFoodServiceStudentsById(string id, string fields, string IfNoneMatch)
        {
            var studentsByIdResponse = _studentsApi.GetStudentsById(id, IfNoneMatch);

            if (!ManualCompositesHelper.IsValidRestResponse(studentsByIdResponse))
                return ManualCompositesHelper.CopyRestResponse<FoodServiceStudent>(studentsByIdResponse, null);

            var mappedResponse = GetManualFoodServiceStudentComposite(new List<Student> { studentsByIdResponse.Data }, studentsByIdResponse);
            return ManualCompositesHelper.CopyRestResponse(mappedResponse, mappedResponse.Data.Single());
        }

        private static string GetSectionKey(FoodServiceStudent_studentSectionAssociation section)
        {
            return section.schoolId + section.classPeriodName + section.localCourseCode +
                   section.termDescriptor + section.schoolYear +
                   section.classroomIdentificationCode + section.uniqueSectionCode +
                   section.sequenceOfCourse;
        }

        private IRestResponse<List<FoodServiceStudent>> GetManualFoodServiceStudentComposite(List<Student> students, IRestResponse originalRestResponse)
        {
            var results = new List<FoodServiceStudent>();
            // Only need memory caches for objects that are at least 2 levels deep
            var sectionStaffLookups = new Dictionary<string, List<string>>();
            var staffLookups = new Dictionary<string, Staff>();
            var parentLookups = new Dictionary<string, Parent>();
            var studentResults = students.Select(s =>
                new FoodServiceStudent
                {
                    studentUniqueId = s.studentUniqueId,
                    firstName = s.firstName,
                    middleName = s.middleName,
                    lastSurname = s.lastSurname,
                    birthDate = s.birthDate,
                    foodServiceStatusDescriptor = s.foodServiceStatusDescriptor,
                    addresses = s.addresses.Select(a =>
                        new FoodServiceStudent_studentAddress
                        {
                            addressType = a.addressType,
                            streetNumberName = a.streetNumberName,
                            apartmentRoomSuiteNumber = a.apartmentRoomSuiteNumber,
                            buildingSiteNumber = a.buildingSiteNumber,
                            city = a.city,
                            stateAbbreviationType = a.stateAbbreviationType,
                            postalCode = a.postalCode
                        }).ToList(),
                    identificationCodes = s.identificationCodes.Select(ic =>
                        new FoodServiceStudent_studentIdentificationCode
                        {
                            assigningOrganizationIdentificationCode = ic.assigningOrganizationIdentificationCode,
                            studentIdentificationSystemDescriptor = ic.studentIdentificationSystemDescriptor,
                            identificationCode = ic.identificationCode
                        }).ToList(),
                    electronicMails = s.electronicMails.Select(em =>
                        new FoodServiceStudent_studentElectronicMail
                        {
                            primaryEmailAddressIndicator = em.primaryEmailAddressIndicator,
                            electronicMailType = em.electronicMailType,
                            electronicMailAddress = em.electronicMailAddress
                        }).ToList(),
                    telephones = s.telephones.Select(t =>
                        new FoodServiceStudent_studentTelephone
                        {
                            telephoneNumberType = t.telephoneNumberType,
                            orderOfPriority = t.orderOfPriority,
                            telephoneNumber = t.telephoneNumber
                        }).ToList()
                }).ToList();
            foreach (var studentResult in studentResults)
            {
                // Get StudentSchoolAssociation records
                var getStudentSchoolAssociationsResponse =
                    _studentSchoolAssociationsApi.GetStudentSchoolAssociationsAll(
                        null, null,
                        studentResult.studentUniqueId,
                        null, null, null, null, null, null, null, null, null, null,
                        null, null, null, null, null, null, null, null, null, null);
                if (!ManualCompositesHelper.IsValidRestResponse(getStudentSchoolAssociationsResponse))
                    return ManualCompositesHelper.CopyRestResponse(getStudentSchoolAssociationsResponse, results);
                studentResult.schools =
                    getStudentSchoolAssociationsResponse.Data.Select(
                        ssa => new FoodServiceStudent_studentSchoolAssociation
                        {
                            entryGradeLevelDescriptor = ssa.entryGradeLevelDescriptor,
                            exitWithdrawDate = ssa.exitWithdrawDate,
                            schoolId = ssa.schoolReference.schoolId
                        }).ToList();

                // Get StudentSectionAssociation records
                var getStudentSectionAssociationsResponse =
                    _studentSectionAssociationsApi.GetStudentSectionAssociationsAll(
                        null, null,
                        studentResult.studentUniqueId,
                        null, null, null, null, null, null, null, null,
                        null, null, null, null, null, null, null);
                if (!ManualCompositesHelper.IsValidRestResponse(getStudentSectionAssociationsResponse))
                    return ManualCompositesHelper.CopyRestResponse(getStudentSectionAssociationsResponse, results);
                studentResult.sections =
                    getStudentSectionAssociationsResponse.Data.Select(
                        ssa => new FoodServiceStudent_studentSectionAssociation
                        {
                            homeroomIndicator = ssa.homeroomIndicator,
                            schoolId = ssa.sectionReference.schoolId,
                            classPeriodName = ssa.sectionReference.classPeriodName,
                            localCourseCode = ssa.sectionReference.localCourseCode,
                            termDescriptor = ssa.sectionReference.termDescriptor,
                            schoolYear = ssa.sectionReference.schoolYear,
                            classroomIdentificationCode = ssa.sectionReference.classroomIdentificationCode,
                            uniqueSectionCode = ssa.sectionReference.uniqueSectionCode,
                            sequenceOfCourse = ssa.sectionReference.sequenceOfCourse,
                            staff = new List<FoodServiceStudent_studentSectionAssociation_staffSectionAssociation>()
                        }).ToList();

                foreach (var section in studentResult.sections)
                {
                    var sectionKey = GetSectionKey(section);
                    if (!sectionStaffLookups.ContainsKey(sectionKey))
                    {
                        // Get the staff associated with the section
                        var getStaffSectionAssociationsResponse = _staffSectionAssociationsApi
                            .GetStaffSectionAssociationsAll(
                                null, null, null,
                                section.classroomIdentificationCode,
                                section.schoolId,
                                section.classPeriodName,
                                section.localCourseCode,
                                section.schoolYear,
                                section.termDescriptor,
                                section.uniqueSectionCode,
                                section.sequenceOfCourse,
                                null, null, null, null, null, null, null);
                        if (!ManualCompositesHelper.IsValidRestResponse(getStaffSectionAssociationsResponse))
                            return ManualCompositesHelper.CopyRestResponse(getStaffSectionAssociationsResponse, results);

                        sectionStaffLookups.Add(sectionKey, getStaffSectionAssociationsResponse.Data.Select(
                            stsa => stsa.staffReference.staffUniqueId).ToList());
                    }


                    // Select the staff ids to lookup them up one by one
                    var staffIds = sectionStaffLookups[sectionKey];
                    foreach (var staffId in staffIds)
                    {
                        if (!staffLookups.ContainsKey(staffId))
                        {
                            var getStaffResponse = _staffsApi.GetStaffByKey(staffId, null);
                            if (!ManualCompositesHelper.IsValidRestResponse(getStaffResponse))
                                return ManualCompositesHelper.CopyRestResponse(getStaffResponse, results);
                            staffLookups.Add(staffId, getStaffResponse.Data);
                        }
                        var staffToAdd = staffLookups[staffId];
                        section.staff.Add(new FoodServiceStudent_studentSectionAssociation_staffSectionAssociation
                        {
                            firstName = staffToAdd.firstName,
                            middleName = staffToAdd.middleName,
                            lastSurname = staffToAdd.lastSurname
                        });
                    }
                }

                // Get StudentParentAssociation records
                var getStudentParentAssociationsResponse =
                    _studentParentAssociationsApi.GetStudentParentAssociationsAll(
                        null, null,
                        studentResult.studentUniqueId,
                        null, null, null, null, null, null, null, null,
                        null, null, null);
                if (!ManualCompositesHelper.IsValidRestResponse(getStudentParentAssociationsResponse))
                    return ManualCompositesHelper.CopyRestResponse(getStudentParentAssociationsResponse, results);

                studentResult.parents = new List<FoodServiceStudent_studentParentAssociation>();
                foreach (var parentAssociation in getStudentParentAssociationsResponse.Data)
                {
                    var parentId = parentAssociation.parentReference.parentUniqueId;
                    if (!parentLookups.ContainsKey(parentId))
                    {
                        var parentResponse = _parentsApi.GetParentByKey(parentAssociation.parentReference.parentUniqueId,
                            null);
                        if (!ManualCompositesHelper.IsValidRestResponse(parentResponse))
                            return ManualCompositesHelper.CopyRestResponse(parentResponse, results);

                        parentLookups.Add(parentId, parentResponse.Data);
                    }
                    var parent = parentLookups[parentId];

                    studentResult.parents.Add(new FoodServiceStudent_studentParentAssociation
                    {
                        firstName = parent.firstName,
                        middleName = parent.middleName,
                        lastSurname = parent.lastSurname,
                        primaryContactStatus = parentAssociation.primaryContactStatus,
                        emergencyContactStatus = parentAssociation.emergencyContactStatus,
                        contactPriority = parentAssociation.contactPriority,
                        contactRestrictions = parentAssociation.contactRestrictions,
                        telephones = parent.telephones.Select(t =>
                            new FoodServiceStudent_studentParentAssociation_parentTelephone
                            {
                                telephoneNumberType = t.telephoneNumberType,
                                orderOfPriority = t.orderOfPriority,
                                telephoneNumber = t.telephoneNumber
                            }).ToList(),
                        electronicMails = parent.electronicMails.Select(em =>
                            new FoodServiceStudent_studentParentAssociation_parentElectronicMail
                            {
                                primaryEmailAddressIndicator = em.primaryEmailAddressIndicator,
                                electronicMailType = em.electronicMailType,
                                electronicMailAddress = em.electronicMailAddress
                            }).ToList()
                    });
                }
            }
            return ManualCompositesHelper.CopyRestResponse(originalRestResponse, studentResults);
        }
    }
}