  
using System;
using System.Collections.Generic;
using System.Linq;
using Michigan.Ods.Sdk.Helpers;
using Michigan.Ods.Sdk.Models;
using Michigan.Ods.Sdk.Models.Composites.Transportation;
using RestSharp;

namespace Michigan.Ods.Sdk.Api.Composites
{
    public class TransportationApi
    {
        private readonly StudentsApi _studentsApi;
        private readonly StudentSchoolAssociationsApi _studentSchoolAssociationsApi;
        private readonly StudentParentAssociationsApi _studentParentAssociationsApi;
        private readonly StudentTransportationsApi _studentTransportationsApi;
        private readonly BusRoutesApi _busRoutesApi;
        private readonly ParentsApi _parentsApi;

        public TransportationApi(IRestClient client)
        {
            _studentsApi = new StudentsApi(client);
            _studentSchoolAssociationsApi = new StudentSchoolAssociationsApi(client);
            _studentParentAssociationsApi = new StudentParentAssociationsApi(client);
            _studentTransportationsApi = new StudentTransportationsApi(client);
            _busRoutesApi = new BusRoutesApi(client);
            _parentsApi = new ParentsApi(client);
        }


        /// <summary>
        ///     Retrieves resources based with paging capabilities (using the &quot;Get All&quot; pattern). This GET operation
        ///     provides access to resources using the &quot;Get All&quot; pattern. In this version of the API there is support for
        ///     paging.
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="fields">
        ///     Specifies a subset of properties that should be returned for each entity (e.g. &quot;
        ///     property1,collection1(collProp1,collProp2)&quot;).
        /// </param>
        /// <returns></returns>
        public IRestResponse<List<TransportationStudent>> GetTransportationStudentsAll(int? offset, int? limit,
            string fields)
        {
            var studentGetAllResponse = _studentsApi.GetStudentsAll(offset, limit);

            if (!ManualCompositesHelper.IsValidRestResponse(studentGetAllResponse))
                return ManualCompositesHelper.CopyRestResponse(studentGetAllResponse, new List<TransportationStudent>());

            return GetManualTransportationStudentComposite(studentGetAllResponse.Data, studentGetAllResponse);
        }

        /// <summary>
        ///     Retrieves a specific resource using the resource's identifier (using the &quot;Get By Example&quot; pattern). This
        ///     GET operation provides access to resources using the &quot;Get By Example&quot; pattern. The values of any
        ///     properties of the resource that are specified will be used to return all matching results (if it exists).
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="fields">
        ///     Specifies a subset of properties that should be returned for each entity (e.g. &quot;
        ///     property1,collection1(collProp1,collProp2)&quot;).
        /// </param>
        /// <param name="q">
        ///     Specifies a query filter expression for the request.  Currently only supports range-based queries on
        ///     dates and numbers (e.g. &quot;[2016-03-07..2016-03-10]&quot;).
        /// </param>
        /// <param name="studentUniqueId">A unique alphanumeric code assigned to a student.</param>
        /// <param name="citizenshipStatusType">
        ///     A unique identifier used as Primary Key, not derived from business logic, when
        ///     acting as Foreign Key, references the parent table.
        /// </param>
        /// <param name="birthCountryDescriptor">
        ///     A unique identifier used as Primary Key, not derived from business logic, when
        ///     acting as Foreign Key, references the parent table.
        /// </param>
        /// <param name="limitedEnglishProficiencyDescriptor">
        ///     An indication that the student has sufficient difficulty speaking,
        ///     reading, writing, or understanding the English language, as to require special English Language services.
        /// </param>
        /// <param name="oldEthnicityType">
        ///     Previous definition of Ethnicity combining Hispanic/latino and race:  1 - American
        ///     Indian or Alaskan Native  2 - Asian or Pacific Islander  3 - Black, not of Hispanic origin  4 - Hispanic  5 -
        ///     White, not of Hispanic origin
        /// </param>
        /// <param name="schoolFoodServicesEligibilityDescriptor">
        ///     A unique identifier used as Primary Key, not derived from
        ///     business logic, when acting as Foreign Key, references the parent table.
        /// </param>
        /// <param name="sexType">A person''s gender.</param>
        /// <param name="birthStateAbbreviationType">
        ///     The abbreviation for the name of the state (within the United States) or
        ///     extra-state jurisdiction in which an individual was born.
        /// </param>
        /// <param name="personalTitlePrefix">A prefix used to denote the title, degree, position, or seniority of the person.</param>
        /// <param name="firstName">
        ///     A name given to an individual at birth, baptism, or during another naming ceremony, or through
        ///     legal change.
        /// </param>
        /// <param name="middleName">A secondary name given to an individual at birth, baptism, or during another naming ceremony.</param>
        /// <param name="lastSurname">The name borne in common by members of a family.</param>
        /// <param name="generationCodeSuffix">
        ///     An appendage, if any, used to denote an individual's generation in his family (e.g.,
        ///     Jr., Sr., III).
        /// </param>
        /// <param name="maidenName">The person's maiden name.</param>
        /// <param name="birthDate">The month, day, and year on which an individual was born.</param>
        /// <param name="birthCity">
        ///     The set of elements that capture relevant data regarding a person's birth, including birth date
        ///     and place of birth.
        /// </param>
        /// <param name="dateEnteredUS">For students born outside of the U.S., the date the student entered the U.S.</param>
        /// <param name="multipleBirthStatus">
        ///     Indicator of whether the student was born with other siblings (i.e., twins, triplets,
        ///     etc.)
        /// </param>
        /// <param name="profileThumbnail">ProfileThumbnail.</param>
        /// <param name="hispanicLatinoEthnicity">
        ///     An indication that the individual traces his or her origin or descent to Mexico,
        ///     Puerto Rico, Cuba, Central, and South America, and other Spanish cultures, regardless of race. The term, &quot;
        ///     Spanish origin,&quot; can be used in addition to &quot;Hispanic or Latino.&quot;
        /// </param>
        /// <param name="economicDisadvantaged">
        ///     An indication of inadequate financial condition of an individual's family, as
        ///     determined by family income, number of family members/dependents, participation in public assistance programs,
        ///     and/or other characteristics considered relevant by federal, state, and local policy.
        /// </param>
        /// <param name="displacementStatus">
        ///     Indicates a state health or weather related event that displaces a group of students,
        ///     and may require additional funding, educational, or social services.
        /// </param>
        /// <param name="loginId">The login ID for the user; used for security access control interface.</param>
        /// <param name="birthInternationalProvince">
        ///     For students born outside of the US, the Province or jurisdiction in which an
        ///     individual is born.
        /// </param>
        /// <param name="yearOfEntry">
        ///     (MI-EXT) The four-digit calendar year in which the student first attended school in the
        ///     United States.
        /// </param>
        /// <param name="raceAsOfDate">(MI-EXT) The date of the most recent change in Race.</param>
        /// <param name="schoolFoodServicesEligibilityAsOfDate">
        ///     (MI-EXT) The date of the most recent change in
        ///     SchoolFoodServicesEligibility.
        /// </param>
        /// <param name="addressAsOfDate">(MI-EXT) The date of the most recent changes to Address.</param>
        /// <param name="homelessDescriptor">
        ///     A unique identifier used as Primary Key, not derived from business logic, when acting
        ///     as Foreign Key, references the parent table.
        /// </param>
        /// <param name="foodServiceStatusDescriptor">
        ///     (MI EXT) A unique identifier used as Primary Key, not derived from business
        ///     logic, when acting as Foreign Key, references the parent table.
        /// </param>
        /// <param name="medicalNotes">
        ///     (MI EXT) Medical notes such as &quot;Allergic to Bee Stings&quot; - For Transportation
        ///     records.
        /// </param>
        /// <returns></returns>
        public IRestResponse<List<TransportationStudent>> GetTransportationStudentsByExample(int? offset, int? limit,
            string fields, string q, string studentUniqueId, string citizenshipStatusType, string birthCountryDescriptor,
            string limitedEnglishProficiencyDescriptor, string oldEthnicityType,
            string schoolFoodServicesEligibilityDescriptor, string sexType, string birthStateAbbreviationType,
            string personalTitlePrefix, string firstName, string middleName, string lastSurname,
            string generationCodeSuffix, string maidenName, DateTime? birthDate, string birthCity,
            DateTime? dateEnteredUS, bool? multipleBirthStatus, string profileThumbnail, bool? hispanicLatinoEthnicity,
            bool? economicDisadvantaged, string displacementStatus, string loginId, string birthInternationalProvince,
            string id, int? yearOfEntry, DateTime? raceAsOfDate, DateTime? schoolFoodServicesEligibilityAsOfDate,
            DateTime? addressAsOfDate, string homelessDescriptor, string foodServiceStatusDescriptor,
            string medicalNotes)
        {
            var studentGetAllResponse = _studentsApi.GetStudentsAll(offset, limit, studentUniqueId, personalTitlePrefix,
                firstName, middleName, lastSurname, generationCodeSuffix, maidenName, sexType, birthDate, birthCity,
                birthStateAbbreviationType, dateEnteredUS, multipleBirthStatus, profileThumbnail,
                hispanicLatinoEthnicity, oldEthnicityType, economicDisadvantaged,
                schoolFoodServicesEligibilityDescriptor, limitedEnglishProficiencyDescriptor, displacementStatus,
                loginId, birthInternationalProvince, citizenshipStatusType, id, birthCountryDescriptor, yearOfEntry,
                raceAsOfDate, schoolFoodServicesEligibilityAsOfDate, addressAsOfDate, homelessDescriptor,
                foodServiceStatusDescriptor, medicalNotes);

            if (!ManualCompositesHelper.IsValidRestResponse(studentGetAllResponse))
                return ManualCompositesHelper.CopyRestResponse(studentGetAllResponse, new List<TransportationStudent>());

            return GetManualTransportationStudentComposite(studentGetAllResponse.Data, studentGetAllResponse);
        }

        /// <summary>
        ///     Retrieves a specific resource using the resource's identifier (using the &quot;Get By Key&quot; pattern). This GET
        ///     operation provides access to resources using the &quot;Get By Key&quot; pattern. The values of the natural key of
        ///     the resource must be fully specified, and the service will return the matching result (if it exists).
        /// </summary>
        /// <param name="studentUniqueId">A unique alphanumeric code assigned to a student.</param>
        /// <param name="fields">
        ///     Specifies a subset of properties that should be returned for each entity (e.g. &quot;
        ///     property1,collection1(collProp1,collProp2)&quot;).
        /// </param>
        /// <param name="IfNoneMatch">
        ///     The previously returned ETag header value, used here to prevent the unnecessary data transfer
        ///     of an unchanged resource.
        /// </param>
        /// <returns></returns>
        public IRestResponse<TransportationStudent> GetTransportationStudentsByKey(string studentUniqueId,
            string fields, string IfNoneMatch)
        {
            var studentGetByKeyResponse = _studentsApi.GetStudentByKey(studentUniqueId, IfNoneMatch);

            if (!ManualCompositesHelper.IsValidRestResponse(studentGetByKeyResponse))
                return ManualCompositesHelper.CopyRestResponse<TransportationStudent>(studentGetByKeyResponse, null);

            var mappedResponse =
                GetManualTransportationStudentComposite(new List<Student> {studentGetByKeyResponse.Data},
                    studentGetByKeyResponse);
            return ManualCompositesHelper.CopyRestResponse(mappedResponse, mappedResponse.Data.Single());
        }

        /// <summary>
        ///     Retrieves a specific resource using the resource's identifier (using the &quot;Get By Id&quot; pattern). This GET
        ///     operation provides access to resources using the &quot;Get By Id&quot; pattern.
        /// </summary>
        /// <param name="id"></param>
        /// <param name="fields">
        ///     Specifies a subset of properties that should be returned for each entity (e.g. &quot;
        ///     property1,collection1(collProp1,collProp2)&quot;).
        /// </param>
        /// <param name="IfNoneMatch">
        ///     The previously returned ETag header value, used here to prevent the unnecessary data transfer
        ///     of an unchanged resource.
        /// </param>
        /// <returns></returns>
        public IRestResponse<TransportationStudent> GetTransportationStudentsById(string id, string fields,
            string IfNoneMatch)
        {
            var studentsByIdResponse = _studentsApi.GetStudentsById(id, IfNoneMatch);

            if (!ManualCompositesHelper.IsValidRestResponse(studentsByIdResponse))
                return ManualCompositesHelper.CopyRestResponse<TransportationStudent>(studentsByIdResponse, null);

            var mappedResponse = GetManualTransportationStudentComposite(new List<Student> {studentsByIdResponse.Data},
                studentsByIdResponse);
            return ManualCompositesHelper.CopyRestResponse(mappedResponse, mappedResponse.Data.Single());
        }

        private IRestResponse<List<TransportationStudent>> GetManualTransportationStudentComposite(
            List<Student> students, IRestResponse originalRestResponse)
        {
            var results = new List<TransportationStudent>();
            // Only need memory caches for objects that are at least 2 levels deep
            var sectionStaffLookups = new Dictionary<string, List<string>>();
            var busRouteLookups = new Dictionary<string, BusRoute>();
            var parentLookups = new Dictionary<string, Parent>();
            var studentResults = students.Select(s =>
                new TransportationStudent
                {
                    studentUniqueId = s.studentUniqueId,
                    firstName = s.firstName,
                    middleName = s.middleName,
                    lastSurname = s.lastSurname,
                    birthDate = s.birthDate,
                    medicalNotes = s.medicalNotes,
                    addresses = s.addresses.Select(a =>
                        new TransportationStudent_studentAddress
                        {
                            addressType = a.addressType,
                            streetNumberName = a.streetNumberName,
                            apartmentRoomSuiteNumber = a.apartmentRoomSuiteNumber,
                            buildingSiteNumber = a.buildingSiteNumber,
                            city = a.city,
                            stateAbbreviationType = a.stateAbbreviationType,
                            postalCode = a.postalCode
                        }).ToList(),
                    identificationCodes = s.identificationCodes.Select(ic =>
                        new TransportationStudent_studentIdentificationCode
                        {
                            assigningOrganizationIdentificationCode = ic.assigningOrganizationIdentificationCode,
                            studentIdentificationSystemDescriptor = ic.studentIdentificationSystemDescriptor,
                            identificationCode = ic.identificationCode
                        }).ToList(),
                    electronicMails = s.electronicMails.Select(em =>
                        new TransportationStudent_studentElectronicMail
                        {
                            primaryEmailAddressIndicator = em.primaryEmailAddressIndicator,
                            electronicMailType = em.electronicMailType,
                            electronicMailAddress = em.electronicMailAddress
                        }).ToList(),
                    telephones = s.telephones.Select(t =>
                        new TransportationStudent_studentTelephone
                        {
                            telephoneNumberType = t.telephoneNumberType,
                            orderOfPriority = t.orderOfPriority,
                            telephoneNumber = t.telephoneNumber
                        }).ToList()
                }).ToList();
            foreach (var studentResult in studentResults)
            {
                // Get StudentSchoolAssociation records
                var getStudentSchoolAssociationsResponse =
                    _studentSchoolAssociationsApi.GetStudentSchoolAssociationsAll(
                        null, null,
                        studentResult.studentUniqueId,
                        null, null, null, null, null, null, null, null, null, null,
                        null, null, null, null, null, null, null, null, null, null);
                if (!ManualCompositesHelper.IsValidRestResponse(getStudentSchoolAssociationsResponse))
                    return ManualCompositesHelper.CopyRestResponse(getStudentSchoolAssociationsResponse, results);
                studentResult.schools =
                    getStudentSchoolAssociationsResponse.Data.Select(
                        ssa => new TransportationStudent_studentSchoolAssociation
                        {
                            entryGradeLevelDescriptor = ssa.entryGradeLevelDescriptor,
                            exitWithdrawDate = ssa.exitWithdrawDate,
                            schoolId = ssa.schoolReference.schoolId
                        }).ToList();

                // Get StudentTransportation records
                var getStudentTransportationsResponse =
                    _studentTransportationsApi.GetStudentTransportationsAll(
                        null, null,
                        studentResult.studentUniqueId,
                        null, null, null, null, null, null);
                if (!ManualCompositesHelper.IsValidRestResponse(getStudentTransportationsResponse))
                    return ManualCompositesHelper.CopyRestResponse(getStudentTransportationsResponse, results);

                studentResult.studentTransportations = getStudentTransportationsResponse.Data.Select(st =>
                    new TransportationStudent_studentTransporation
                    {
                        schoolId = st.schoolReference.schoolId,
                        busRouteTime = st.busRouteTime,
                        busNumber = st.busRouteReference.busNumber,
                        directionDescriptor = st.busRouteReference.directionDescriptor,
                        dayOfWeekDescriptor = st.busRouteReference.dayOfWeekDescriptor
                    }).ToList();
                foreach (var studentTransportation in studentResult.studentTransportations)
                {
                    var busRouteKey = GetBusRouteKey(studentTransportation);
                    if (!busRouteLookups.ContainsKey(busRouteKey))
                    {
                        var busRouteResponse = _busRoutesApi.GetBusRouteByKey(
                            studentTransportation.busNumber,
                            studentTransportation.directionDescriptor,
                            studentTransportation.dayOfWeekDescriptor,
                            null);
                        if (!ManualCompositesHelper.IsValidRestResponse(busRouteResponse))
                            return ManualCompositesHelper.CopyRestResponse(busRouteResponse, results);

                        busRouteLookups.Add(busRouteKey, busRouteResponse.Data);
                    }
                    var busRoute = busRouteLookups[busRouteKey];

                    studentTransportation.startTime = busRoute.startTime;
                    studentTransportation.endTime = busRoute.endTime;
                    studentTransportation.name = busRoute.name;
                    studentTransportation.busRouteNumberOrId = busRoute.busRouteNumberOrId;
                }

                // Get StudentParentAssociation records
                var getStudentParentAssociationsResponse =
                    _studentParentAssociationsApi.GetStudentParentAssociationsAll(
                        null, null,
                        studentResult.studentUniqueId,
                        null, null, null, null, null, null, null, null,
                        null, null, null);
                if (!ManualCompositesHelper.IsValidRestResponse(getStudentParentAssociationsResponse))
                    return ManualCompositesHelper.CopyRestResponse(getStudentParentAssociationsResponse, results);

                studentResult.parents = new List<TransportationStudent_studentParentAssociation>();
                foreach (var parentAssociation in getStudentParentAssociationsResponse.Data)
                {
                    var parentId = parentAssociation.parentReference.parentUniqueId;
                    if (!parentLookups.ContainsKey(parentId))
                    {
                        var parentResponse = _parentsApi.GetParentByKey(
                            parentAssociation.parentReference.parentUniqueId,
                            null);
                        if (!ManualCompositesHelper.IsValidRestResponse(parentResponse))
                            return ManualCompositesHelper.CopyRestResponse(parentResponse, results);

                        parentLookups.Add(parentId, parentResponse.Data);
                    }
                    var parent = parentLookups[parentId];

                    studentResult.parents.Add(new TransportationStudent_studentParentAssociation
                    {
                        firstName = parent.firstName,
                        middleName = parent.middleName,
                        lastSurname = parent.lastSurname,
                        primaryContactStatus = parentAssociation.primaryContactStatus,
                        emergencyContactStatus = parentAssociation.emergencyContactStatus,
                        contactPriority = parentAssociation.contactPriority,
                        contactRestrictions = parentAssociation.contactRestrictions,
                        householdId = parentAssociation.householdId,
                        telephones = parent.telephones.Select(t =>
                            new TransportationStudent_studentParentAssociation_parentTelephone
                            {
                                telephoneNumberType = t.telephoneNumberType,
                                orderOfPriority = t.orderOfPriority,
                                telephoneNumber = t.telephoneNumber
                            }).ToList(),
                        electronicMails = parent.electronicMails.Select(em =>
                            new TransportationStudent_studentParentAssociation_parentElectronicMail
                            {
                                primaryEmailAddressIndicator = em.primaryEmailAddressIndicator,
                                electronicMailType = em.electronicMailType,
                                electronicMailAddress = em.electronicMailAddress
                            }).ToList()
                    });
                }
            }
            return ManualCompositesHelper.CopyRestResponse(originalRestResponse, studentResults);
        }

        private static string GetBusRouteKey(TransportationStudent_studentTransporation studentTransportation)
        {
            return studentTransportation.busNumber + studentTransportation.directionDescriptor +
                   studentTransportation.dayOfWeekDescriptor;
        }
    }
}
