using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Models {
  public class DisciplineAction {
    /* The unique identifier of the resource. */
    public string id { get; set; }

    /* A reference to the related School resource. */
    public SchoolReference assignmentSchoolReference { get; set; }

    /* A reference to the related School resource. */
    public SchoolReference responsibilitySchoolReference { get; set; }

    /* A reference to the related Student resource. */
    public StudentReference studentReference { get; set; }

    /* Identifier assigned by the education organization to the discipline action. */
    public string identifier { get; set; }

    /* The date of the DisciplineAction. */
    public DateTime? disciplineDate { get; set; }

    /* The length of time in school days for the Discipline Action (e.g. removal, detention), if applicable. */
    public int? length { get; set; }

    /* Indicates the actual length in school days of a student's disciplinary assignment. */
    public int? actualDisciplineActionLength { get; set; }

    /* Key for DisciplineActionLengthDifferenceReason */
    public string lengthDifferenceReasonType { get; set; }

    /* An indication of whether or not this disciplinary action taken against a student was imposed as a consequence of state or local zero tolerance policies. */
    public bool? relatedToZeroTolerancePolicy { get; set; }

    /* An unordered collection of disciplineActionDisciplines.  Type of action, such as removal from the classroom, used to discipline the student involved as a perpetrator in a discipline incident. */
    public List<DisciplineActionDiscipline> disciplines { get; set; }

    /* An unordered collection of disciplineActionDisciplineIncidents.   */
    public List<DisciplineActionDisciplineIncident> disciplineIncidents { get; set; }

    /* An unordered collection of disciplineActionStaffs.  The staff responsible for enforcing the discipline action. */
    public List<DisciplineActionStaff> staffs { get; set; }

    /* A unique system-generated value that identifies the version of the resource. */
    public string _etag { get; set; }

    }
}
