using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Models {
  public class DisciplineIncident {
    /* The unique identifier of the resource. */
    public string id { get; set; }

    /* A reference to the related School resource. */
    public SchoolReference schoolReference { get; set; }

    /* A reference to the related Staff resource. */
    public StaffReference staffReference { get; set; }

    /* A locally assigned unique identifier (within the school or school district) to identify each specific incident or occurrence. The same identifier should be used to document the entire incident even if it included multiple offenses and multiple offenders. */
    public string incidentIdentifier { get; set; }

    /* The month, day, and year on which the DisciplineIncident occurred. */
    public DateTime? incidentDate { get; set; }

    /* An indication of the time of day the incident took place. */
    public string incidentTime { get; set; }

    /* Identifies where the incident occurred and whether or not it occurred on campus, for example:  On campus  Administrative offices area  Cafeteria area  Classroom  Hallway or stairs  ... */
    public string incidentLocationType { get; set; }

    /* The description for an incident. */
    public string incidentDescription { get; set; }

    /* Information on the type of individual who reported the incident. When known and/or if useful, use a more specific option code (e.g., "Counselor" rather than "Professional Staff"); for example:Student  Parent/guardian  Law enforcement officer  Nonschool personnel  Representative of visiting school  ... */
    public string reporterDescriptionDescriptor { get; set; }

    /* Identifies the reporter of the incident by name. */
    public string reporterName { get; set; }

    /* Indicator of whether the incident was reported to law enforcement. */
    public bool? reportedToLawEnforcement { get; set; }

    /* The case number assigned to the incident by law enforcement or other organization. */
    public string caseNumber { get; set; }

    /* The value of any quantifiable monetary loss directly resulting from the DisciplineIncident. Examples include the value of repairs necessitated by vandalism of a school facility, or the value of personnel resources used for repairs or consumed by the incident. */
    public double? incidentCost { get; set; }

    /* Indicate if when the Incident Type involved physical violence with injury and the injury inflicted meets the definition of Serious Bodily Injury per the United States Code paragragh  (3) of subsection (h) of section 1365 of title 18:  "(3) the term  serious bodily injury  means bodily injury which involves   (A) a substantial risk of death;  (B) extreme physical pain;  (C) protracted and obvious disfigurement; or  (D) protracted loss or impairment of the function of a bodily member, organ, or  mental faculty;"   0=No, 1=Yes */
    public bool? seriousBodilyInjury { get; set; }

    /* Indicate if an incident of the Incident Type involved physical violence involved oral, anal, or  vaginal penetration forcibly or against the person's will or where the victim is incapable of giving consent. Includes rape, fondling, indecent liberties, child molestation, and sodomy.  0=No, 1=Yes */
    public bool? sexualAssault { get; set; }

    /* An unordered collection of disciplineIncidentBehaviors.  The categories of behavior describing a discipline incident. */
    public List<DisciplineIncidentBehavior> behaviors { get; set; }

    /* An unordered collection of disciplineIncidentWeapons.  Identifies the type of weapon used during an incident. */
    public List<DisciplineIncidentWeapon> weapons { get; set; }

    /* A unique system-generated value that identifies the version of the resource. */
    public string _etag { get; set; }

    }
}
