using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Models {
  public class GraduationPlan {
    /* The unique identifier of the resource. */
    public string id { get; set; }

    /* A reference to the related EducationOrganization resource. */
    public EducationOrganizationReference educationOrganizationReference { get; set; }

    /* A reference to the related SchoolYearType resource. */
    public SchoolYearTypeReference graduationSchoolYearTypeReference { get; set; }

    /* A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table. */
    public string typeDescriptor { get; set; }

    /* An indicator of whether the GraduationPlan is tailored for an individual. */
    public bool? individualPlan { get; set; }

    /* The total number of credits required for graduation under this plan. */
    public double? totalRequiredCredits { get; set; }

    /* Key for Credit */
    public string totalRequiredCreditType { get; set; }

    /* Conversion factor that when multiplied by the number of credits is equivalent to Carnegie units. */
    public double? totalRequiredCreditConversion { get; set; }

    /* (MI-EXT) A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table. */
    public string curriculumDescriptor { get; set; }

    /* The most recent date of changes to CreditModification. */
    public DateTime? creditModificationAsOfDate { get; set; }

    /* An unordered collection of graduationPlanCreditModifications.  (MI-EXT) To indicate the academic area that is being modified by the personal curriculum. */
    public List<GraduationPlanCreditModification> creditModifications { get; set; }

    /* An unordered collection of graduationPlanCreditsByCourses.  The total credits required for graduation by taking a specific course, or by taking one or more from a set of courses. */
    public List<GraduationPlanCreditsByCourse> creditsByCourses { get; set; }

    /* An unordered collection of graduationPlanCreditsBySubjects.  The total number of credits required in a subject to graduate.  Only those courses identified as a high school course requirement are eligible to meet subject credit requirements. */
    public List<GraduationPlanCreditsBySubject> creditsBySubjects { get; set; }

    /* An unordered collection of graduationPlanRequiredAssessments.  The total credits required for graduation by taking a specific course, or by taking one or more from a set of courses. */
    public List<GraduationPlanRequiredAssessment> requiredAssessments { get; set; }

    /* A unique system-generated value that identifies the version of the resource. */
    public string _etag { get; set; }

    }
}
