using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Models {
  public class Intervention {
    /* The unique identifier of the resource. */
    public string id { get; set; }

    /* A reference to the related EducationOrganization resource. */
    public EducationOrganizationReference educationOrganizationReference { get; set; }

    /* A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table. */
    public string identificationCode { get; set; }

    /* A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table. */
    public string classType { get; set; }

    /* A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table. */
    public string deliveryMethodType { get; set; }

    /* The start date for the intervention implementation. */
    public DateTime? beginDate { get; set; }

    /* The end date for the intervention implementation. */
    public DateTime? endDate { get; set; }

    /* An unordered collection of interventionAppropriateGradeLevels.  Grade levels for the intervention. */
    public List<InterventionAppropriateGradeLevel> appropriateGradeLevels { get; set; }

    /* An unordered collection of interventionAppropriateSexes.  Gender(s) for which the intervention is appropriate. */
    public List<InterventionAppropriateSex> appropriateSexes { get; set; }

    /* An unordered collection of interventionDiagnoses.  Targeted purpose of the intervention (e.g., attendance issue, dropout risk). */
    public List<InterventionDiagnosis> diagnoses { get; set; }

    /* An unordered collection of interventionEducationContents.  Resources related to or used in this intervention, including any documentation around the intervention prescription itself. Since an intervention prescription is intended to be a published intervention, an intervention prescription should have at least one such resource. */
    public List<InterventionEducationContent> educationContents { get; set; }

    /* An unordered collection of interventionInterventionPrescriptions.  The reference to the intervention prescription being followed in this intervention implementation. */
    public List<InterventionInterventionPrescription> interventionPrescriptions { get; set; }

    /* An unordered collection of interventionLearningResourceMetadataURIs.  Resources related to or used in this intervention, including any documentation around the intervention prescription itself. Since an intervention prescription is intended to be a published intervention, an intervention prescription should have at least one such resource. */
    public List<InterventionLearningResourceMetadataURI> learningResourceMetadataURIs { get; set; }

    /* An unordered collection of interventionMeetingTimes.  The times at which this intervention is scheduled to meet. */
    public List<InterventionMeetingTime> meetingTimes { get; set; }

    /* An unordered collection of interventionPopulationServeds.  A subset of students that are the focus of the intervention.  */
    public List<InterventionPopulationServed> populationServeds { get; set; }

    /* An unordered collection of interventionStaffs.  Staff member associated with the intervention. */
    public List<InterventionStaff> staffs { get; set; }

    /* An unordered collection of interventionURIs.  Resources related to or used in this intervention, including any documentation around the intervention prescription itself. Since an intervention prescription is intended to be a published intervention, an intervention prescription should have at least one such resource. */
    public List<InterventionURI> uris { get; set; }

    /* A unique system-generated value that identifies the version of the resource. */
    public string _etag { get; set; }

    }
}
