using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Models {
  public class Parent {
    /* The unique identifier of the resource. */
    public string id { get; set; }

    /* A unique alphanumeric code assigned to a parent. */
    public string parentUniqueId { get; set; }

    /* A prefix used to denote the title, degree, position, or seniority of the person. */
    public string personalTitlePrefix { get; set; }

    /* A name given to an individual at birth, baptism, or during another naming ceremony, or through legal change. */
    public string firstName { get; set; }

    /* A secondary name given to an individual at birth, baptism, or during another naming ceremony. */
    public string middleName { get; set; }

    /* The name borne in common by members of a family. */
    public string lastSurname { get; set; }

    /* An appendage, if any, used to denote an individual's generation in his family (e.g., Jr., Sr., III). */
    public string generationCodeSuffix { get; set; }

    /* The person's maiden name. */
    public string maidenName { get; set; }

    /* A person''s gender. */
    public string sexType { get; set; }

    /* The login ID for the user; used for security access control interface. */
    public string loginId { get; set; }

    /* An unordered collection of parentAddresses.  Parent's address, if different from the student address. */
    public List<ParentAddress> addresses { get; set; }

    /* An unordered collection of parentElectronicMails.  The numbers, letters and symbols used to identify an electronic mail (e-mail) user within the network to which the individual or organization belongs. */
    public List<ParentElectronicMail> electronicMails { get; set; }

    /* An unordered collection of parentIdentificationDocuments.  This type represents the valid document that a person uses for identification. */
    public List<ParentIdentificationDocument> identificationDocuments { get; set; }

    /* An unordered collection of parentInternationalAddresses.  Parent's address, if different from the student address. */
    public List<ParentInternationalAddress> internationalAddresses { get; set; }

    /* An unordered collection of parentOtherNames.  Other names (e.g., alias, nickname, previous legal name) associated with a person. */
    public List<ParentOtherName> otherNames { get; set; }

    /* An unordered collection of parentTelephones.  The 10-digit telephone number, including the area code, for the person. */
    public List<ParentTelephone> telephones { get; set; }

    /* A unique system-generated value that identifies the version of the resource. */
    public string _etag { get; set; }

    }
}
