using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Models {
  public class Student {
    /* The unique identifier of the resource. */
    public string id { get; set; }

    /* A unique alphanumeric code assigned to a student. */
    public string studentUniqueId { get; set; }

    /* A prefix used to denote the title, degree, position, or seniority of the person. */
    public string personalTitlePrefix { get; set; }

    /* A name given to an individual at birth, baptism, or during another naming ceremony, or through legal change. */
    public string firstName { get; set; }

    /* A secondary name given to an individual at birth, baptism, or during another naming ceremony. */
    public string middleName { get; set; }

    /* The name borne in common by members of a family. */
    public string lastSurname { get; set; }

    /* An appendage, if any, used to denote an individual's generation in his family (e.g., Jr., Sr., III). */
    public string generationCodeSuffix { get; set; }

    /* The person's maiden name. */
    public string maidenName { get; set; }

    /* A person''s gender. */
    public string sexType { get; set; }

    /* The month, day, and year on which an individual was born. */
    public DateTime? birthDate { get; set; }

    /* The set of elements that capture relevant data regarding a person's birth, including birth date and place of birth. */
    public string birthCity { get; set; }

    /* The abbreviation for the name of the state (within the United States) or extra-state jurisdiction in which an individual was born. */
    public string birthStateAbbreviationType { get; set; }

    /* For students born outside of the U.S., the date the student entered the U.S. */
    public DateTime? dateEnteredUS { get; set; }

    /* Indicator of whether the student was born with other siblings (i.e., twins, triplets, etc.) */
    public bool? multipleBirthStatus { get; set; }

    /* ProfileThumbnail. */
    public string profileThumbnail { get; set; }

    /* An indication that the individual traces his or her origin or descent to Mexico, Puerto Rico, Cuba, Central, and South America, and other Spanish cultures, regardless of race. The term, "Spanish origin," can be used in addition to "Hispanic or Latino." */
    public bool? hispanicLatinoEthnicity { get; set; }

    /* Previous definition of Ethnicity combining Hispanic/latino and race:  1 - American Indian or Alaskan Native  2 - Asian or Pacific Islander  3 - Black, not of Hispanic origin  4 - Hispanic  5 - White, not of Hispanic origin */
    public string oldEthnicityType { get; set; }

    /* An indication of inadequate financial condition of an individual's family, as determined by family income, number of family members/dependents, participation in public assistance programs, and/or other characteristics considered relevant by federal, state, and local policy. */
    public bool? economicDisadvantaged { get; set; }

    /* A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table. */
    public string schoolFoodServicesEligibilityDescriptor { get; set; }

    /* An indication that the student has sufficient difficulty speaking, reading, writing, or understanding the English language, as to require special English Language services. */
    public string limitedEnglishProficiencyDescriptor { get; set; }

    /* Indicates a state health or weather related event that displaces a group of students, and may require additional funding, educational, or social services. */
    public string displacementStatus { get; set; }

    /* The login ID for the user; used for security access control interface. */
    public string loginId { get; set; }

    /* For students born outside of the US, the Province or jurisdiction in which an individual is born. */
    public string birthInternationalProvince { get; set; }

    /* A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table. */
    public string citizenshipStatusType { get; set; }

    /* A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table. */
    public string birthCountryDescriptor { get; set; }

    /* (MI-EXT) The four-digit calendar year in which the student first attended school in the United States. */
    public int? yearOfEntry { get; set; }

    /* (MI-EXT) The date of the most recent change in Race. */
    public DateTime? raceAsOfDate { get; set; }

    /* (MI-EXT) The date of the most recent change in SchoolFoodServicesEligibility. */
    public DateTime? schoolFoodServicesEligibilityAsOfDate { get; set; }

    /* (MI-EXT) The date of the most recent changes to Address. */
    public DateTime? addressAsOfDate { get; set; }

    /* A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table. */
    public string homelessDescriptor { get; set; }

    /* (MI EXT) A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table. */
    public string foodServiceStatusDescriptor { get; set; }

    /* (MI EXT) Medical notes such as "Allergic to Bee Stings" - For Transportation records. */
    public string medicalNotes { get; set; }

    /* The student's relative preference to visual, auditory and tactile learning expressed as percentages. */
    public StudentLearningStyle learningStyle { get; set; }

    /* An unordered collection of studentAddresses.  The set of elements that describes an address, including the street address, city, state, and ZIP code. */
    public List<StudentAddress> addresses { get; set; }

    /* An unordered collection of studentCharacteristics.  Reflects important characteristics of the student's home situation: Displaced Homemaker, Homeless, Immigrant, Migratory, Military Parent, Pregnant Teen, Single Parent, Unaccompanied Youth. */
    public List<StudentCharacteristic> characteristics { get; set; }

    /* An unordered collection of studentCohortYears.  The type and year of a cohort (e.g., 9th grade) the student belongs to as determined by the year that student entered a specific grade. */
    public List<StudentCohortYear> cohortYears { get; set; }

    /* An unordered collection of studentDisabilities.  This type represents an impairment of body structure or function, a limitation in activities or a restriction in participation, as ordered by severity of impairment. */
    public List<StudentDisability> disabilities { get; set; }

    /* An unordered collection of studentElectronicMails.   */
    public List<StudentElectronicMail> electronicMails { get; set; }

    /* An unordered collection of studentIdentificationCodes.  A coding scheme that is used for identification and record-keeping purposes by schools, social services or other agencies to refer to a student. */
    public List<StudentIdentificationCode> identificationCodes { get; set; }

    /* An unordered collection of studentIdentificationDocuments.  Represents the valid document that a person uses for identification. */
    public List<StudentIdentificationDocument> identificationDocuments { get; set; }

    /* An unordered collection of studentIndicators.  An indicator or metric computed for the student (e.g., at risk) to influence more effective education or direct specific interventions. */
    public List<StudentIndicator> indicators { get; set; }

    /* An unordered collection of studentInternationalAddresses.  The set of elements that describes an address, including the street address and country for international students. */
    public List<StudentInternationalAddress> internationalAddresses { get; set; }

    /* An unordered collection of studentLanguages.  Language(s) the individual uses to communicate. */
    public List<StudentLanguage> languages { get; set; }

    /* An unordered collection of studentMSDSRaceOrders.  (MI EXT) Captures the racial group and order choice for MSDS. */
    public List<StudentMSDSRaceOrder> msdsRaceOrders { get; set; }

    /* An unordered collection of studentOtherNames.  Other names (e.g., alias, nickname, previous legal name) associated with a person. */
    public List<StudentOtherName> otherNames { get; set; }

    /* An unordered collection of studentProgramParticipations.  Key programs the student is participating in or receives services from. */
    public List<StudentProgramParticipation> programParticipations { get; set; }

    /* An unordered collection of studentRaces.  The general racial category which most clearly reflects the individual's recognition of his or her community or with which the individual most identifies. The data model allows for multiple entries so that each individual can specify all appropriate races. */
    public List<StudentRace> races { get; set; }

    /* An unordered collection of studentTelephones.  The 10-digit telephone number, including the area code, for the person. */
    public List<StudentTelephone> telephones { get; set; }

    /* An unordered collection of studentVisas.  Describe the types of Visa that a non-U.S. citizen student holds. */
    public List<StudentVisa> visas { get; set; }

    /* A unique system-generated value that identifies the version of the resource. */
    public string _etag { get; set; }

    }
}
